﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using InChatter.Service.Data;

namespace InChatter.Service
{
    public class Chat : IChat
    {
        public static Dictionary<string, IChatCallback> Callbacks = new Dictionary<string, IChatCallback>();
        public bool Login(string clientId)
        {
            if (Callbacks.Keys.Contains(clientId))
            {
                return false;
            }
            IChatCallback callback;
            //告知其他用户,新用户上线
            BroadcastUserState(clientId, true);
            callback = OperationContext.Current.GetCallbackChannel<IChatCallback>();
            Callbacks.Add(clientId, callback);
            //返回在线客户端
            return true;
        }

        public string GetOnlineClient()
        {
            IChatCallback callback = OperationContext.Current.GetCallbackChannel<IChatCallback>();
            //获取除本客户端以外的其他在线客户端
            var result = Callbacks.Where(p => p.Value != callback).Select(p => p.Key).ToList();
            return string.Join(",", result);
        }

        public void SendMsg(Data.InChatterMessage message)
        {
            if (message == null)
            {
                return;
            }
            if (message.Type == "notice")
            {
                SendNotice(message);
            }
            else if (message.Type == "msg")
            {
                try
                {
                    Callbacks[message.ReceiverID].ReceiveMsg(message);
                }
                catch
                {
                    Logout(message.ReceiverID);
                }
            }
        }

        public void Logout(string clientId)
        {
            try
            {
                lock (Callbacks)
                {
                    //移除退出用户的callBacks
                    Callbacks.Remove(clientId);
                }
                //广播下线消息
                BroadcastUserState(clientId, false);
            }
            catch
            {

            }
        }

        /// <summary>
        /// 通知其他用户上线或者下线消息
        /// </summary>
        /// <param name="employeeId"></param>
        /// <param name="isLogin"></param>
        private void BroadcastUserState(string clientId, bool isLogin)
        {
            List<string> list = new List<string>();
            foreach (var item in Callbacks)
            {
                InChatterMessage txt = new InChatterMessage();
                txt.Content = "";
                txt.SenderID = clientId;
                txt.ReceiverID = item.Key;
                if (isLogin)
                {
                    txt.Type = "logon";
                }
                else
                {
                    txt.Type = "logoff";
                }
                txt.SendTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                try
                {
                    item.Value.ReceiveMsg(txt);
                }
                catch
                {
                    list.Add(item.Key);
                }
            }
            RemoveDisconnectCallBacks(list);
        }

        /// <summary>
        /// 发送通知消息，接受者为所有在线客户
        /// </summary>
        /// <param name="msg"></param>
        private void SendNotice(InChatterMessage msg)
        {
            List<string> list = new List<string>();
            foreach (var item in Callbacks)
            {
                try
                {
                    item.Value.ReceiveMsg(msg);
                }
                catch
                {
                    list.Add(item.Key);
                }
            }
            RemoveDisconnectCallBacks(list);
        }

        /// <summary>
        /// 移除断开连接客户端
        /// </summary>
        /// <param name="list"></param>
        private void RemoveDisconnectCallBacks(List<string> list)
        {
            if (list.Count > 0)
            {
                //移除出错的callBack
                foreach (var item in list)
                {
                    Callbacks.Remove(item);
                }
                //将连接通道出错的项认定为下线,发送下线消息给在线客户端
                foreach (var item in list)
                {
                    BroadcastUserState(item, false);
                }
            }
        }
    }
}
